/*	ccdrawable.h
	All your base are belong to us!
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		ccdrawable.h
 *	@brief		Declaration of the <code>ccDrawable</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */

#ifndef __CCDRAWABLE_H
#define __CCDRAWABLE_H

#include <ccsys.h>
#include <ccvideo.h>

namespace crabclaw	{
	/**
	 *	Drawable base class.
	 *	This class is the base-class of all objects that can be drawn in a
	 *	<code>ccSceneManager</code>.
	 *	@since	0.9.0
	 */
	class ccDrawable	{
		public:
			/**
			 *	Virtual destructor.
			 *	Doesn't do anything of note.
			 *	@since			0.9.0
			 */
			virtual ~ccDrawable()	{	};
			
			/**
			 *	Drawing function.
			 *	Pure virtual (abstract) function for drawing to a <code>ccScreen</code>.
			 *	@param	scr 	The <code>ccScreen</code> object to draw to.
			 *	@since			0.9.0
			 */
			virtual void draw(ccScreen *scr)=0;
			
			/**
			 *	List identifying function.
			 *	Pure virtual (abstract) function to determine the primitive list
			 *	that this <code>ccDrawable</code> should be drawn in.
			 *	@return			The list to be drawn in.
			 *	@since			0.9.0
			 *	@see			ccScreen::LIST_OPAQUE
			 *	@see			ccScreen::LIST_TRANSLUCENT
			 */
			virtual int getList()=0;
	};
}

#endif
