/*	cctimer.h
	Timing class
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
/**
 *	@file		cctimer.h
 *	@brief		Declaration of the <code>ccTimer</code> class.
 *	@author		Lawrence Sebald
 *	@date		2004
 *	@since		0.9.0
 *	@version	0.9.0
 */
 
#ifndef __CCTIMER_H
#define __CCTIMER_H

#include <ccsys.h>
#include <time.h>

namespace crabclaw	{
	/**
	 *	Simple time class.
	 *	This class is meant to serve as a wrapper around native time functions.
	 *	@since	0.9.0
	 */
	class ccTimer	{
		private:
			ccTimer()	{	};
		public:
			/**
			 *	Get the system-dependent milisecond count.
			 *	@return			Milisecond time.
			 *	@since			0.9.0
			 */
			static uint64 getMilliseconds();
			
			/**
			 *	Get the time.
			 *	Get the time represented as seconds after 1/1/1970 12:00 AM.
			 *	This is the same as a call to ::time(NULL).
			 *	@return			Current time.
			 *	@since			0.9.0
			 */
			static time_t getTime();
			
	};
}

#endif
