/*	ccline.cc
	ccLine implementation
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccline.h>

namespace crabclaw	{
	ccLine::ccLine(int lst, int x1, int y1, int x2, int y2, int z, int thick, uint32 c, int ty)	{
		list = lst;
		xp1 = x1;
		xp2 = x2;
		yp1 = y1;
		yp2 = y2;
		zp = z;
		thickness = thick;
		col = c;
		t = ty;
	}
	
	ccLine::~ccLine()	{
	}
	
	ccLine *ccLine::createHorizontalLine(int lst, int x1, int x2, int y, int z, int thick, uint32 c)	{
		return new ccLine(lst, x1, y, x2, y, z, thick, c, 0);
	}
	
	ccLine *ccLine::createVerticalLine(int lst, int x, int y1, int y2, int z, int thick, uint32 c)	{
		return new ccLine(lst, x, y1, x, y2, z, thick, c, 1);
	}
	
	ccLine *ccLine::createDiagonalLine(int lst, int x1, int y1, int x2, int y2, int z, int thick, uint32 c)	{
		return new ccLine(lst, x1, y1, x2, y2, z, thick, c, 2);
	}
	
	void ccLine::draw(ccScreen *scr)	{
		int c;
		
		scr->beginPoly();
		if(t == 0)	{
			if(xp1 > xp2)	{
				c = xp1;
				xp1 = xp2;
				xp2 = c;
			}
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1, yp1 + (thickness >> 1), zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1, yp1 - (thickness >> 1), zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp2, yp1 + (thickness >> 1), zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::END_VERT, xp2, yp1 - (thickness >> 1), zp, 0, 0, col));
		}
		else if(t == 1)	{
			if(yp1 > yp2)	{
				c = yp1;
				yp1 = yp2;
				yp2 = c;
			}
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1 - (thickness >> 1), yp2, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1 - (thickness >> 1), yp1, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1 + (thickness >> 1), yp2, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::END_VERT, xp1 + (thickness >> 1), yp1, zp, 0, 0, col));
		}
		else	{
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1 - (thickness >> 1), yp1, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp2 - (thickness >> 1), yp2, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xp1 + (thickness >> 1), yp1, zp, 0, 0, col));
			scr->submitVertex(ccVertex(ccVertex::END_VERT, xp2 + (thickness >> 1), yp2, zp, 0, 0, col));
		}
		scr->finishPoly();
	}
}
