/*	ccpolygon.cc
	Drawable Polygons
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccpolygon.h>
#include <ccvideo.h>
#include <ccvertex.h>
#include <cclist.h>

namespace crabclaw	{
	ccPolygon::ccPolygon(uint8 listtype)	{
		list = listtype;
		vert = new ccList<ccVertex>();
	}
	
	void ccPolygon::draw(ccScreen *scr)	{
		ccListNode<ccVertex> *n = vert->getFirstNode();
		scr->beginPoly();
		while(n != NULL)	{
			scr->submitVertex(*n->getData());
			n = n->getNext();
		}
		scr->finishPoly();
	}
	
	ccMovingPoly::ccMovingPoly(uint8 listtype, void (*mvfunc)(ccMovingPoly&)) : ccPolygon(listtype)	{
		mv = mvfunc;
	}
	
	void ccMovingPoly::draw(ccScreen *scr)	{
		ccPolygon::draw(scr);
		if(mv != NULL)	{
			mv(*this);
		}
	}
	
	void ccMovingPoly::setVertex(int num, ccVertex *v)	{
		ccListNode<ccVertex> *n = vert->getFirstNode();
		int i;
		
		for(i = 0; i < num; i++)	{
			n = n->getNext();
		}
		
		n->setData(v);
	}
	
	void ccMovingPoly::move(int x, int y, int z)	{
		ccListNode<ccVertex> *n = vert->getFirstNode();
		ccVertex *t;
		
		while(n)	{
			t = n->getData();
			
			t->setX(t->getX() + x);
			t->setY(t->getY() + y);
			t->setZ(t->getZ() + z);
			
			n = n->getNext();
		}
	}
	
	ccTexturedPoly::ccTexturedPoly(int listtype, ccTexture *txrptr) : ccPolygon(listtype)	{
		txr = txrptr;
	}
	
	void ccTexturedPoly::draw(ccScreen *scr)	{
		ccListNode<ccVertex> *n = vert->getFirstNode();
		txr->submit(scr);
		while(n != NULL)	{
			scr->submitVertex(*n->getData());
			n = n->getNext();
		}
		scr->finishPoly();
	}
}
