/*	ccsprite.cc
	Drawable Sprites
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccsprite.h>
#include <ccvideo.h>
#include <ccvertex.h>
#include <cclist.h>

namespace crabclaw	{
	ccSprite::ccSprite(ccTexture *tex, float u, float v, uint8 listtype)	{
		list = listtype;
		txr = tex;
		uf = u;
		vf = v;
		sp_x = (int) (1.0f / u);
		sp_y = (int) (1.0f / v);
		xpos = ypos = zpos = 5;
		argb = 0xFFFFFFFF;
		oargb = 0;
		px = 0;
		py = 0;
		dw = dh = 200;		
	}
	
	ccSprite::~ccSprite()	{
		delete txr;
	}
	
	void ccSprite::setPos(int x, int y, int z)	{
		xpos = x;
		ypos = y;
		zpos = z;
	}
	
	void ccSprite::setColors(uint32 a, uint32 o)	{
		argb = a;
		oargb = o;
	}
	
	void ccSprite::setDimensions(int w, int h)	{
		dw = w;
		dh = h;
	}
	
	void ccSprite::setDisplayed(int x, int y)	{
		px = x;
		py = y;
	}
	
	void ccSprite::draw(ccScreen *scr)	{
		txr->submit(scr);
		scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xpos, ypos + dh, zpos, uf * px, vf * (py + 1), argb, oargb));
		scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xpos, ypos, zpos, uf * px, vf * py, argb, oargb));
		scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, xpos + dw, ypos + dh, zpos, uf * (px + 1), vf * (py + 1), argb, oargb));
		scr->submitVertex(ccVertex(ccVertex::END_VERT, xpos + dw, ypos, zpos, uf * (px + 1), vf * py, argb, oargb));
		scr->finishPoly();
	}
}
