/*	ccscenemgr.cc
	Simple scene management
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccscenemgr.h>
#include <time.h>

namespace crabclaw	{
	ccSceneManager::ccSceneManager(void (*func)(ccSceneManager*), ccScreen * scr)	{
		callback = func;
		screen = scr;
		op_list = new ccList<ccDrawable>();
		tr_list = new ccList<ccDrawable>();
		sttime = time(NULL);
		fcount = 0;
		enabled = true;
	}
	
	ccSceneManager::~ccSceneManager()	{
		delete op_list;
		delete tr_list;
	}
	
	void ccSceneManager::add(ccDrawable *obj)	{
		if(obj->getList() == ccScreen::LIST_OPAQUE)	{
			op_list->add(obj);
		}
		else	{
			tr_list->add(obj);
		}
	}
	
	float ccSceneManager::getFPS()	{
		return ((float) fcount) / (time(NULL) - sttime);
	}
	
	void ccSceneManager::run()	{
		ccListNode<ccDrawable> *n;
		ccDrawable *tmp;
		
		while(enabled)	{
			fcount++;
			screen->beginScene();
		
			screen->beginList(ccScreen::LIST_OPAQUE);
			n = op_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				tmp->draw(screen);
				n = n->getNext();
			}
			screen->finishList();
			
			screen->beginList(ccScreen::LIST_TRANSLUCENT);
			n = tr_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				tmp->draw(screen);
				n = n->getNext();
			}
			screen->finishList();
			
			screen->finishScene();
			if(callback != NULL)
				callback(this);
		}
	}
	
	void ccSceneManager::stop()	{
		enabled = false;
	}
	
	void ccSceneManager::remove(uint8 list, int num)	{
		if(list == ccScreen::LIST_OPAQUE)	{
			op_list->remove(num);
		}
		else	{
			tr_list->remove(num);
		}
	}
	
	void ccSceneManager::remove(ccDrawable *obj)	{
		ccListNode<ccDrawable> *n;
		
		if(obj->getList() == ccScreen::LIST_OPAQUE)	{
			n = op_list->getFirstNode();
		}
		else	{
			n = tr_list->getFirstNode();
		}
		
		while(n != NULL)	{
			if(n->getData() == obj)	{
				n->getPrev()->setNext(n->getNext());
				n->getNext()->setPrev(n->getPrev());
				delete n;
				break;
			}
			n = n->getNext();
		}
	}
}
