/*	ccfont.cc
	Font functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccfont.h>

namespace crabclaw	{
	#define LOAD4444(r, g, b, a) (((a>>4)<<12)|((r>>4)<<8)|((g>>4)<<4)|((b>>4)))

	static inline int __next_p2(int a)	{
		int r = 8;
		while(r < a)
			r <<= 1;
	
		return r;
	}
	
	static inline void __copy_bitmap(FT_Face face, uint16 * data, int w, int h)	{
		int i, j;
		FT_Bitmap bmp = face->glyph->bitmap;
	
		memset(data, 0, w * h * 2);
	
		for(i = 0; i < bmp.rows; i++)	{
			for(j = 0; j < bmp.width; j++)	{
				data[j + i * w] = LOAD4444(bmp.buffer[j + bmp.width * i], bmp.buffer[j + bmp.width * i], bmp.buffer[j + bmp.width * i], bmp.buffer[j + bmp.width * i]);
			}
		}
	}
	
	ccTTFFont::ccTTFFont(char *fn, float he)	{
		FT_Face face;
		int i;
		char * data;
		int w, h;
	
		FT_Init_FreeType(&lib);
		
		if(FT_New_Face(lib, fn, 0, &face))	{
			::printf("Error loading font!\n");
			return;
		}

		FT_Set_Char_Size(face, (int) (he * 64),  (int) (he * 64), 96, 96);
		height = he;
		
		txr = (ccTexture **) malloc(sizeof(ccTexture *) * 224);
		glyph = (ttf_glyph_t *) malloc(sizeof(ttf_glyph_t) * 224);
		
		for(i = 32; i < 256; i++)	{
			if(FT_Load_Char(face, i, FT_LOAD_RENDER))	{
				::printf("Unable to load character %c\n", i);
				continue;
			}
	
			w = __next_p2(face->glyph->bitmap.width);
			h = __next_p2(face->glyph->bitmap.rows);
			glyph[i - 32].w = face->glyph->bitmap.width;
			glyph[i - 32].h = face->glyph->bitmap.rows;
			glyph[i - 32].u = (float) face->glyph->bitmap.width / w;
			glyph[i - 32].v = (float) face->glyph->bitmap.rows / h;
			glyph[i - 32].below = (face->glyph->metrics.height -
			                       face->glyph->metrics.horiBearingY) / 64.0f;
			glyph[i - 32].adv = (float) face->glyph->metrics.horiAdvance / 64;
			glyph[i - 32].advb = (face->glyph->metrics.horiBearingX / 64);
			data = (char *) malloc(w * h * 2);
	
			__copy_bitmap(face, (uint16 *) data, w, h);
			
			txr[i - 32] = new ccTexture(data, w, h, ccTexture::TXRFMT_ARGB4444);
			free(data);
		}

		FT_Done_Face(face);
	}
	
	ccTTFFont::~ccTTFFont()	{
		int i;
		for(i = 0; i < 224; i++)	{
			delete txr[i];
		}
		free(txr);
		free(glyph);
	}
	
	void ccTTFFont::drawString(ccScreen *scr, const char *str, int x, int y, float he, uint32 argb)	{
		int i, num;
		float h, u, v, factor, adv;
		int w, hi, off, ox, below, advb;
	
		y -= (int) (he - 2);
		ox = x;
		factor = he / height;
		hi = (int) (he - (he / 3));
	
	
		for(i = 0; str[i] != '\0'; i++)	{
			if(str[i] == '\n')	{
				y += (int) (he + 2);
				x = (int) ox;
			}
			else if(str[i] == ' ')	{
				x += (int) (0.5f * he);
			}
			else	{
				num = str[i] - 32;
				below = (int) (glyph[num].below * factor);
				w = (int) (glyph[num].w * factor);
				h = (glyph[num].h) * factor - below;
				u = glyph[num].u;
				v = glyph[num].v;
				adv = glyph[num].adv * factor;
				off = (int) (hi - h);
				advb = (int) (glyph[num].advb * factor);
	
				txr[num]->submit(scr);
				scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x + advb, y + hi + below, 35, 0.0f, v, argb));
				scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x + advb, y + off, 35, 0.0f, 0.0f, argb));
				scr->submitVertex(ccVertex(ccVertex::NORMAL_VERT, x + advb + w, y + hi + below, 35, u, v, argb));
				scr->submitVertex(ccVertex(ccVertex::END_VERT, x + advb + w, y + off, 35, u, 0.0f, argb));
				scr->finishPoly();
				x += (int) adv;
			}
		}
	}
	void ccTTFFont::printf(ccScreen *scr, int x, int y, float he, uint32 argb, const char *fmt, ...)	{
		char str[1024];
		va_list ap;
	
		if(fmt == NULL)	{
			return;
		}
	
		va_start(ap, fmt);
		vsprintf(str, fmt, ap);
		va_end(ap);
	
		drawString(scr, str, x, y, he, argb);
	}
	
	ccTextString::ccTextString(ccTTFFont *font, char *string, float height, uint32 color)	{
		str = string;
		f = font;
		this->height = height;
		argb = color;
	}
	
	ccTextString::~ccTextString()	{
	}
	
	void ccTextString::setPosition(int xp, int yp)	{
		x = xp;
		y = yp;
	}
	
	void ccTextString::draw(ccScreen *scr)	{
		f->drawString(scr, str, x, y, height, argb);
	}
}
