/*	ccmenu.cc
	Generic Menus
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccmenu.h>
	
namespace crabclaw	{
	ccMenuItem::ccMenuItem(ccDrawable *unsel, ccDrawable *sel, bool freed)	{
		o_unsel = unsel;
		o_sel = sel;
		fdraw = freed;
	}
	
	ccMenuItem::~ccMenuItem()	{
		if(fdraw)	{
			delete o_unsel;
			delete o_sel;
		}
	}
	
	ccDrawable *ccMenuItem::getUnsel()	{
		return o_unsel;
	}
	
	ccDrawable *ccMenuItem::getSel()	{
		return o_sel;
	}
	
	void ccMenu::init(ccScreen *s, int initial_sel, void (*eh)(int))	{
		scr = s;
		selected = initial_sel;
		event_hnd = eh;
		it_list = new ccList<ccMenuItem>();
		o_list = new ccList<ccDrawable>();
		stopped = false;
	}
	
	void ccMenu::deinit()	{
		delete it_list;
		delete o_list;
	}
	
	ccMenu::ccMenu(ccScreen *scr, int initial_sel, void (*eh)(int))	{
		init(scr, initial_sel, eh);
	}
	
	ccMenu::~ccMenu()	{
		deinit();
	}
	
	void ccMenu::addItem(ccMenuItem *mi)	{
		it_list->add(mi);
	}
	
	void ccMenu::addObject(ccDrawable *dr)	{
		o_list->add(dr);
	}
	
	void ccMenu::setSelected(int sel)	{
		selected = sel;
	}
	
	void ccMenu::stop()	{
		stopped = true;
	}
	
	void ccMenu::run()	{
		int i = 0;
		ccListNode<ccDrawable> *n;
		ccDrawable *tmp;
		ccListNode<ccMenuItem> *n2;
		ccMenuItem *tmp2;
		
		while(!stopped)	{
			scr->beginScene();
			scr->beginList(ccScreen::LIST_OPAQUE);
			
			i = 0;
			n = o_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				if(tmp->getList() == ccScreen::LIST_OPAQUE)
					tmp->draw(scr);
				n = n->getNext();
			}
			
			n2 = it_list->getFirstNode();			
			while(n2 != NULL)	{
				tmp2 = n2->getData();
				if(i == selected)
					tmp = tmp2->getSel();
				else
					tmp = tmp2->getUnsel();
				if(tmp->getList() == ccScreen::LIST_OPAQUE)
					tmp->draw(scr);
				n2 = n2->getNext();
				i++;
			}
			
			i = 0;
			scr->finishList();
			scr->beginList(ccScreen::LIST_TRANSLUCENT);
			
			n = o_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				if(tmp->getList() == ccScreen::LIST_TRANSLUCENT)
					tmp->draw(scr);
				n = n->getNext();
			}
			
			n2 = it_list->getFirstNode();			
			while(n2 != NULL)	{
				tmp2 = n2->getData();
				if(i == selected)
					tmp = tmp2->getSel();
				else
					tmp = tmp2->getUnsel();
				if(tmp->getList() == ccScreen::LIST_TRANSLUCENT)
					tmp->draw(scr);
				n2 = n2->getNext();
				i++;
			}
			
			scr->finishList();
			scr->finishScene();
			
			event_hnd(selected);
		}
	}
}
