/*	ccselmenu.cc
	ccSelectorMenu implementation
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccmenu.h>

namespace crabclaw	{
	
	ccSelectorMenu::ccSelectorMenu(ccScreen *s, ccMovingPoly *sel, int initsel, int xo, int yo, int x_c, int y_c, void (*eh)(int, void*), void *param)	{
		scr = s;
		selector = sel;
		this->sel = initsel;
		oldsel = 0;
		xoff = xo;
		yoff = yo;
		dx = x_c;
		dy = y_c;
		event_hnd = eh;
		stopped = true;
		p = param;
		
		o_list = new ccList<ccDrawable>();
	}
	
	ccSelectorMenu::~ccSelectorMenu()	{
		delete o_list;
		delete selector;
	}
	
	void ccSelectorMenu::run()	{
		int i = 0;
		ccListNode<ccDrawable> *n;
		ccDrawable *tmp;
		
		stopped = false;
		
		while(!stopped)	{
			if(oldsel != sel)	{
				selector->move((sel - oldsel) * dx, (sel - oldsel) * dy, 0);
				oldsel = sel;
			}
			
			scr->beginScene();
			scr->beginList(ccScreen::LIST_OPAQUE);
			
			i = 0;
			n = o_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				if(tmp->getList() == ccScreen::LIST_OPAQUE)
					tmp->draw(scr);
				n = n->getNext();
			}
			
			if(selector->getList() == ccScreen::LIST_OPAQUE)	{
				selector->draw(scr);
			}
			
			i = 0;
			scr->finishList();
			scr->beginList(ccScreen::LIST_TRANSLUCENT);
			
			n = o_list->getFirstNode();
			while(n != NULL)	{
				tmp = n->getData();
				if(tmp->getList() == ccScreen::LIST_TRANSLUCENT)
					tmp->draw(scr);
				n = n->getNext();
			}
			
			if(selector->getList() == ccScreen::LIST_TRANSLUCENT)	{
				selector->draw(scr);
			}
			
			scr->finishList();
			scr->finishScene();
			
			event_hnd(sel, p);
		}
	}
	
	void ccSelectorMenu::stop()	{
		stopped = true;
	}
	
	void ccSelectorMenu::add(ccDrawable *dr)	{
		o_list->add(dr);
	}
	
	
	void ccSelectorMenu::incrementSelected()	{
		sel++;
	}
	
	void ccSelectorMenu::decrementSelected()	{
		if(sel != 0)	{
			sel--;
		}
	}
	
	void ccSelectorMenu::setSelected(int sel)	{
		this->sel = sel;
	}
	
	void ccSelectorMenu::setSelector(ccMovingPoly *sel)	{
		selector = sel;
	}
	
	void ccSelectorMenu::setYOffset(int y)	{
		yoff = y;
	}
	
	void ccSelectorMenu::setXOffset(int x)	{
		xoff = x;
	}
	
	void ccSelectorMenu::setDeltaY(int y)	{
		dy = y;
	}
	
	void ccSelectorMenu::setDeltaX(int x)	{
		dx = x;
	}
	
	void ccSelectorMenu::setEventHandler(void (*eh)(int, void *))	{
		event_hnd = eh;
	}
}
