/*	dctexture.cc
	Dreamcast texture functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <cctexture.h>
#include <dc/pvr.h>

namespace crabclaw	{
	ccTexture::ccTexture(void *tex, int w, int h, int fmt)	{
		pvr_poly_cxt_t cxt;
		
		char *pt = (char *) tex;
		int val = ((int) pt) & 0x0FFFFFFF;
		void *txr = tex;
			
		if(val < 0x04000000 || val > 0x07FFFFFF)	{ /* Not in vram */
			txr = pvr_mem_malloc(w * h * 2);
			pvr_txr_load(pt, txr, w * h * 2);
			fmt |= PVR_TXRFMT_NONTWIDDLED;
		}			
		
		pvr_poly_cxt_txr(&cxt, PVR_LIST_OP_POLY, fmt, w, h, txr, PVR_FILTER_NONE);
		pvr_poly_compile(&hdr_op, &cxt);
		
		pvr_poly_cxt_txr(&cxt, PVR_LIST_TR_POLY, fmt, w, h, txr, PVR_FILTER_NONE);
		pvr_poly_compile(&hdr_tr, &cxt);
		
		ptr = txr;
		this->w = w;
		this->h = h;
	}
	
	ccTexture::~ccTexture()	{	
		pvr_mem_free(ptr);
	};
	
	void ccTexture::submit(ccScreen *scr)	{
		/* We only support one ccScreen on the dreamcast, so this is quite easy */
		uint8 list = scr->getOpenList();
		switch(list)	{
			case ccScreen::LIST_OPAQUE:
				pvr_prim(&hdr_op, sizeof(hdr_op));
				break;
			case ccScreen::LIST_TRANSLUCENT:
				pvr_prim(&hdr_tr, sizeof(hdr_tr));
				break;
			default:
				break;
		}
	}
}
		