/*	dcvideo.cc
	Dreamcast video functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
	
#include <ccvideo.h>
#include <dc/video.h>
#include <dc/pvr.h>

namespace crabclaw	{	
	void ccScreen::initScreen()	{
		int dm;
		if(w == 640 && h == 480)	{
			dm = DM_640x480;
		}
		else if(w == 320 && h == 240)	{
			dm = DM_320x240;
		}
		else	{
			printf("Display mode not supported, defaulting to 640x480\n");
			dm = DM_640x480;
		}
		
		/* we only deal with 16-bit color modes... */
		vid_init(dm, PM_RGB565);
		
		pvr_init_defaults();
	}
	
	void ccScreen::destroyScreen()	{
	}
	
	void ccScreen::beginScene()	{
		pvr_wait_ready();
		pvr_scene_begin();
	}
	
	void ccScreen::finishScene()	{
		pvr_scene_finish();
	}
	
	void ccScreen::beginList(uint8 list)	{
		if(list == LIST_OPAQUE)	{
			pvr_list_begin(PVR_LIST_OP_POLY);
			open_list = LIST_OPAQUE;
		}
		else if(list == LIST_TRANSLUCENT)	{
			pvr_list_begin(PVR_LIST_TR_POLY);
			open_list = LIST_TRANSLUCENT;
		}
		else	{
			printf("Unsupported list: %d\n", (int) list);
		}
	}
	
	void ccScreen::finishList()	{
		if(open_list)	{
			pvr_list_finish();
			open_list = 0;
		}
	}
	
	void ccScreen::beginPoly()	{
		pvr_poly_hdr_t hdr;
		pvr_poly_cxt_t cxt;

		pvr_poly_cxt_col(&cxt, open_list);
		pvr_poly_compile(&hdr, &cxt);
		
		pvr_prim(&hdr, sizeof(hdr));		
	}
	
	void ccScreen::finishPoly()	{
	}
	
	void ccScreen::submitVertex(ccVertex v)	{
		pvr_vertex_t vert;
		vert.flags = v.getFlags();
		vert.x = v.getX();
		vert.y = v.getY();
		vert.z = v.getZ();
		vert.u = v.getU();
		vert.v = v.getV();
		vert.argb = v.getARGB();
		vert.oargb = v.getOARGB();
		
		pvr_prim(&vert, sizeof(vert));
	}
}
